dhl_api_url = 'https://xmlpi-ea.dhl.com/XMLdeliveryServlet'
message_dhl_url_down = 'DHL API is not accessible at this moment. Please try after few minutes.'
message_max_response_time = 'Maximum execution time to call DHL API passed. DHL API not able to response within given time.'
message_quote_success = 'DHL quotes returned.'
message_delivery_success = 'DHL delivery created.'
message_pickup_success = 'DHL Pickup created.'

message_request_data_bad_format = 'The data you have sent is not well formated. It looks like some key-value pair are missing. Please check the documentation and send proper request data.'
